if defined KS_SETTINGS   call "%KS_SETTINGS%"

::if not defined KEYSTORE	echo "KEYSTORE is not defined!" & exit /b 1
::if not defined STOREPASS	echo "STOREPASS is not defined!" & exit /b 1

::SetLocal EnableDelayedExpansion
::	set KEYSTORE="%KEYSTORE%"
::	set STOREPASS="%STOREPASS%"
	
::	echo Calling :%*
	call :%*
::EndLocal
exit /b %ERRORLEVEL%



:signCSR [pkey alias] [CSR file] [validity] [output CERT file] 
SetLocal EnableDelayedExpansion
	set ALIAS=%1
	set CSRFILE=%~2
	set VALIDITY=%3
	set CERTFILE=%4
	set DNAME=%5
	set EXT=%~6
	
	if not defined VALIDITY		set VALIDITY="1095"
	if not defined CERTFILE		set CERTFILE="%~n2.cer"
	if     defined DNAME		set DNAME=-dname %DNAME%
	
	
	echo ----------------------------------------
	echo -- Signing '%CSRFILE%' request with '%ALIAS%' key and generating cert chain for DN '%DNAME%', expiring in '%VALIDITY%' days into '%CERTFILE%' ...
	echo ----------------------------------------
	"%KEYTOOL%" -keystore %KEYSTORE% -storepass %STOREPASS% -gencert -rfc -alias %ALIAS% -validity %VALIDITY% -infile "%CSRFILE%" %DNAME% %EXT% > %CERTFILE%
	"%KEYTOOL%" -keystore %KEYSTORE% -storepass %STOREPASS% -exportcert -rfc -alias %ALIAS% >> %CERTFILE%

	echo.
	echo -- DONE --
	echo.
	echo.

EndLocal
exit /b


:exportCert [alias] [certfile] 
SetLocal EnableDelayedExpansion
	set ALIAS=%1
	set CERTFILE=%2
	if not defined CERTFILE 	set CERTFILE="%~1.cer"

	echo ----------------------------------------
	echo -- Exporting '%ALIAS% certificate into '%CERTFILE%' ...
	echo ----------------------------------------
				
	"%KEYTOOL%"	-keystore %KEYSTORE% -storepass %STOREPASS% -exportcert -rfc -alias %ALIAS% > %CERTFILE%
	echo.
	echo -- DONE --
	echo.
	echo.

EndLocal
exit /b


:generateKey [alias] [dname] [validity] [ext]
SetLocal EnableDelayedExpansion
	set ALIAS=%1
	set DNAME=%2
	set VALIDITY=%3
	set EXT=%~4

	echo ----------------------------------------
	echo -- Generating key as '%ALIAS%' in '%KEYSTORE%' with DNAME '%DNAME%' and validity of '%VALIDITY%' ...
	echo ----------------------------------------

	"%KEYTOOL%"	-keystore %KEYSTORE% -storepass %STOREPASS% -keypass %STOREPASS% -genkeypair -keyalg RSA -keysize 2048 ^
				-alias %ALIAS% -dname %DNAME% -validity %VALIDITY% %EXT%
	echo.
	echo -- DONE --
	echo.
	echo.

EndLocal
exit /b


:generateCSR [alias] [keyusage] [ext] [csrfile]
SetLocal EnableDelayedExpansion
	set ALIAS=%1
	set DNAME=%2
	set VALIDITY=%3
	set EXT=%~4
	set CSRFILE=%5
	if not defined CSRFILE 	set CSRFILE="%~1.csr"
	
	
	echo -------------------- --------------------
	echo -- Generating CSR for '%ALIAS%' in '%KEYSTORE%' ...
	echo ----------------------------------------
				
	"%KEYTOOL%" -keystore %KEYSTORE% -storepass %STOREPASS% -certreq -keyalg RSA -alias %ALIAS% -file %CSRFILE% -dname %DNAME% -validity %VALIDITY% %EXT%
	echo.
	echo -- DONE --
	echo.
	echo.

EndLocal
exit /b


:printCSR [csrfile] 
SetLocal EnableDelayedExpansion
	set CSRFILE=%1

	echo ----------------------------------------
	echo -- Content of CSR file ...
	echo ----------------------------------------
				
	"%KEYTOOL%" -printcertreq -file %CSRFILE%
	echo.
	echo -- DONE --
	echo.
	echo.

EndLocal
exit /b


:printCert [cerfile] 
SetLocal EnableDelayedExpansion
	set CERFILE=%1

	echo ----------------------------------------
	echo -- Content of CER file ...
	echo ----------------------------------------
				
	"%KEYTOOL%" -printcert -file %CERFILE%
	echo.
	echo -- DONE --
	echo.
	echo.

EndLocal
exit /b


:renameKey [fromAlias] [toAlias]
SetLocal EnableDelayedExpansion
	set FROM_ALIAS=%1
	set TO_ALIAS=%2

	echo ----------------------------------------
	echo -- Renaming key '%FROM_ALIAS%' to '%TO_ALIAS%' ...
	echo ----------------------------------------
	"%KEYTOOL%" -keystore %KEYSTORE% -storepass %STOREPASS% -keypass %STOREPASS% -changealias -alias %FROM_ALIAS% -destalias %TO_ALIAS%
	echo.
	echo -- DONE --
	echo.
	echo.

EndLocal
exit /b


:deleteKey [alias]
SetLocal EnableDelayedExpansion
	set ALIAS=%1

	echo ----------------------------------------
	echo -- Deleting key '%ALIAS%' ...
	echo ----------------------------------------
	"%KEYTOOL%" -keystore %KEYSTORE% -storepass %STOREPASS% -keypass %STOREPASS% -delete -alias %ALIAS%
	echo.
	echo -- DONE --
	echo.
	echo.

EndLocal
exit /b


:importKeystore [src keystore] [src keystore type]
SetLocal EnableDelayedExpansion
	set SRC_KEYSTORE=%1
	set SRC_STOREPASS=%2
	set SRC_STORETYPE=%3
	if not defined SRC_STORETYPE 	set SRC_STORETYPE="PKCS12"

	echo ----------------------------------------
	echo -- Importing all keys from '%SRC_KEYSTORE% in '%SRC_STORETYPE%' format ...
	echo ----------------------------------------
	"%KEYTOOL%" -importkeystore -noprompt -srckeystore %SRC_KEYSTORE% -srcstorepass %SRC_STOREPASS%  -srcstoretype %SRC_STORETYPE% -destkeystore %KEYSTORE% -deststorepass %STOREPASS%
	echo.
	echo -- DONE --
	echo.
	echo.

EndLocal
exit /b


:exportKeystore [dest keystore] [dest keystore type]
SetLocal EnableDelayedExpansion
	set DEST_KEYSTORE=%1
	set DEST_STORETYPE=%2
	if not defined DEST_STORETYPE 	set DEST_STORETYPE="PKCS12"

	echo ----------------------------------------
	echo -- Exporting all keys to '%DEST_KEYSTORE% in '%DEST_STORETYPE%' format ...
	echo ----------------------------------------
	del "%DEST_KEYSTORE%"
	"%KEYTOOL%" -importkeystore -noprompt -srckeystore %KEYSTORE% -srcstorepass %STOREPASS% -destkeystore %DEST_KEYSTORE% -deststorepass %STOREPASS% -deststoretype %DEST_STORETYPE%
	echo.
	echo -- DONE --
	echo.
	echo.

EndLocal
exit /b


:listCerts
SetLocal EnableDelayedExpansion
	echo ----------------------------------------
	echo -- '%KEYSTORE%' certificates ...
	echo ----------------------------------------
				
	"%KEYTOOL%" -storepass %STOREPASS% -keystore %KEYSTORE% -list 
	echo.
	echo -- DONE --
	echo.
	echo.

EndLocal
exit /b


:printDetails [alias]
SetLocal EnableDelayedExpansion
	set ALIAS=%1

	echo ----------------------------------------
	echo -- '%ALIAS%' certificate details ...
	echo ----------------------------------------
				
	"%KEYTOOL%" -storepass %STOREPASS% -keystore %KEYSTORE% -list -v -alias %ALIAS%
	echo.
	echo -- DONE --
	echo.
	echo.

EndLocal
exit /b


:: Parameters
::	%1 - alias
::	%2 - cert name
::	%3 - Fingerprint
:importCert [alias] [cert filename] [FP]
SetLocal EnableDelayedExpansion
	set ALIAS=%1
	set CERT=%2
	set FP=%3
	if not defined CERT 	set CERT="%~1.cer"
	
	echo ----------------------------------------
	echo -- Importing Certificate '%CERT%' as '%ALIAS%' into '%KEYSTORE%' ...
	echo ----------------------------------------

	"%KEYTOOL%" -importcert -file %CERT% -noprompt -trustcacerts -alias %ALIAS% -storepass %STOREPASS% -keystore %KEYSTORE%
	echo.
	echo -- DONE --
	echo.
	echo.
	
	if defined FP (
		echo ----------------------------------------
		echo -- Checking keystore for [%ALIAS%] key with fingerprint [%FP%] ...
		echo ----------------------------------------

		"%KEYTOOL%" -list -alias %ALIAS% -storepass %STOREPASS% -keystore %KEYSTORE% | find /I /C "%FP: =:%" > nul 2>&1
		if errorlevel 1 ( echo FINGERPRINT DID _NOT_ MATCH!
		) else            echo FINGERPRINT MATCHED! 
	)

EndLocal
exit /b


:signKey [alias] [dname] [ext]
SetLocal EnableDelayedExpansion
	set ALIAS=%1
	set DNAME=%2
	set VALIDITY=%3
	set EXT=%~4
	set CERTFILE="%~1.cer"
	
	
	echo -------------------- --------------------
	echo -- Signing key '%ALIAS%' with '%ROOTCA_ALIAS%' ...
	echo ----------------------------------------
				
	"%KEYTOOL%" -keystore %KEYSTORE% -storepass %STOREPASS% -certreq -keyalg RSA -alias %ALIAS% -validity %VALIDITY% %EXT% | "%KEYTOOL%" -keystore %KEYSTORE% -storepass %STOREPASS% -gencert -rfc -alias "%ROOTCA_ALIAS%" -validity %VALIDITY% %EXT% > %CERTFILE%
	"%KEYTOOL%" -keystore %KEYSTORE% -storepass %STOREPASS% -exportcert -rfc -alias "%ROOTCA_ALIAS%" >> %CERTFILE%
	"%KEYTOOL%" -importcert -file %CERTFILE% -noprompt -trustcacerts -alias %ALIAS% -storepass %STOREPASS% -keystore %KEYSTORE%
	
	echo.
	echo -- DONE --
	echo.
	echo.

EndLocal
exit /b


:changePassword [alias]
SetLocal EnableDelayedExpansion
	set ALIAS=%1
	set OLD_PWD=%2
	set NEW_PWD=%3

	echo ----------------------------------------
	echo -- Changing keystore and alias password...
	echo ----------------------------------------
			
	"%KEYTOOL%" -storepass %OLD_PWD% -keystore %KEYSTORE% -keypasswd   -new %NEW_PWD% -keypass %OLD_PWD% -alias %ALIAS%
	"%KEYTOOL%" -storepass %OLD_PWD% -keystore %KEYSTORE% -storepasswd -new %NEW_PWD%
	
	echo.
	echo -- DONE --
	echo.
	echo.

EndLocal
exit /b


:changeKSPassword [alias]
SetLocal EnableDelayedExpansion
	set OLD_PWD=%1
	set NEW_PWD=%2

	echo ----------------------------------------
	echo -- Changing keystore password...
	echo ----------------------------------------
			
	"%KEYTOOL%" -storepass %OLD_PWD% -keystore %KEYSTORE% -storepasswd -new %NEW_PWD%
	
	echo.
	echo -- DONE --
	echo.
	echo.

EndLocal
exit /b


:sourceFile [file] [reload]
::SetLocal EnableDelayedExpansion
	set srcFile=%1
	set reload=%2
	if defined reload (
		call "%~dp0\%srcFile%.cmd"
		set _src_%srcFile%=true
	) else (
		if not defined _src_%srcFile% (
			call "%~dp0\%srcFile%.cmd"
			set _src_%srcFile%=true
		)
	)

::EndLocal
exit /b


:getFileName [fqfn] [fn]
	set "%~2=%~n1"
exit /b

:getPath [fqfn] [path]
	set "%~2=%~dp1"
exit /b

:getFQFN [relPath] [fqfn]
	set "%~2=%~f1"
exit /b
	
:getName [fqfn] [fn]
	set "%~2=%~n1"
exit /b
	